"""
Bug: it doesn't print if you run 1000 or 2000 ms.

This does the full thing:  it views from the retina, through V1,
to pyramid detection, then sets a fact on.  It also has the planning in.
So, it does the commands that are activated.  This can be set in the
short version in runNoCallBack, or all can be tested with the RunCallBack
version.
"""

#imports for PyNN
from pyNN.utility import get_script_args, normalized_filename
from neo.io import PyNNTextIO

#imports for image reading
import numpy
from PIL import Image
#Other NEAL files and functions
from retinaConnections import set3x3OnConnections,set3x3OffConnections
from retinaConnections import set6x6OnConnections, set6x6OffConnections
from retinaConnections import set9x9OnConnections,set9x9OffConnections
from v1Connections import set6x6OnToHEdgeBottomConnections, set6x6OffToHEdgeBottomConnections
from v1Connections import set6x6OnToSEdgeLeftConnections
from v1Connections import set6x6OnToBEdgeRightConnections
from v1Connections import set3x3OnToAndAngleConnections,set9x9OffToAndAngleConnections
from v1Connections import set3x3OnToLessThanAngleConnections,set9x9OffToLessThanAngleConnections
from v1Connections import set3x3OnToGreaterThanAngleConnections,set9x9OffToGreaterThanAngleConnections
from v1Connections import set6x6OffToHEdgeTopConnections, set9x9OffToHEdgeTopConnections
from v1Connections import set9x9OffToSEdgeRightConnections
from v1Connections import set9x9OffToBEdgeLeftConnections
from v1Connections import set3x3OnToOrAngleConnections, set9x9OffToOrAngleConnections
from objRecConnections import connectCA
from objRecConnections import connectHEdgeBottomToPyramid, connectSEdgeLeftToPyramid
from objRecConnections import connectBEdgeRightToPyramid, connectAndAngleToPyramid
from objRecConnections import connectLessThanAngleToPyramid, connectGreaterThanAngleToPyramid
from objRecConnections import connectHEdgeTopToStalactite, connectSEdgeRightToStalactite
from objRecConnections import connectBEdgeLeftToStalactite, connectOrAngleToStalactite


INPUT_NEURONS_HEIGHT = 50
INPUT_NEURONS_WIDTH = 50

DELAY = 1.0
output_filename='cabot1_out.dat'
SIM_LENGTH = 300.0
#SIM_LENGTH = 2000.0 #For the full plan test

CA_HEIGHT = 10
CA_WIDTH = 10


#------set up object to fact connections----------
#-every neuron of the left 20 of the object turns on the left flag
def leftObjectTurnsLeftFactOn ():
    synWeight = 3.0
    connector = []
    for cRow in range (0,INPUT_NEURONS_HEIGHT):
        for cCol in range (0,int(INPUT_NEURONS_WIDTH*0.4)):
            fromNeuron=cRow*INPUT_NEURONS_WIDTH+cCol
            toNeuron = cCol % 5
            connector = connector + [(fromNeuron,toNeuron,synWeight,DELAY)]
    return connector
    
#-every neuron of the right 20 of the object turns on the left flag
def rightObjectTurnsLeftFactOn ():
    synWeight = 3.0
    connector = []
    for cRow in range (0,INPUT_NEURONS_HEIGHT):
        for cCol in range (int(INPUT_NEURONS_WIDTH*0.6),INPUT_NEURONS_WIDTH):
            fromNeuron=cRow*INPUT_NEURONS_WIDTH+cCol
            toNeuron = cCol % 5
            connector = connector + [(fromNeuron,toNeuron,synWeight,DELAY)]
    return connector
    
#-every neuron of the object turns on the fact
def objectTurnsObjectFactOn ():
    synWeight = 3.0
    connector = []
    for cRow in range (0,INPUT_NEURONS_HEIGHT):
        for cCol in range (0,int(INPUT_NEURONS_WIDTH)):
            fromNeuron=cRow*INPUT_NEURONS_WIDTH+cCol
            toNeuron = cCol % 5
            connector = connector + [(fromNeuron,toNeuron,synWeight,DELAY)]
    return connector
    


#----------------functions for setting plan connections
def setStandardOscillatorConnector(oscPart1,oscPart2,weight):
	connect(oscPart1,oscPart2,weight, DELAY)

def setPlanConnections():
    weight = 6.0
    synWeight = 6.0
    inhibitW = -5.25
    synWeightCoeff = 0.25

    #connections for goals
    setStandardOscillatorConnector(goalTL_1,goalTL_2,weight)
    setStandardOscillatorConnector(goalTL_2,goalTL_1,weight)
    setStandardOscillatorConnector(goalTR_1,goalTR_2,weight)
    setStandardOscillatorConnector(goalTR_2,goalTR_1,weight)
    setStandardOscillatorConnector(goalBack_1,goalBack_2,weight)
    setStandardOscillatorConnector(goalBack_2,goalBack_1,weight)
    setStandardOscillatorConnector(goalForward_1,goalForward_2,weight)
    setStandardOscillatorConnector(goalForward_2,goalForward_1,weight)
    setStandardOscillatorConnector(goal_move_left1_1,goal_move_left1_2,weight)
    setStandardOscillatorConnector(goal_move_left1_2,goal_move_left1_1,weight)
    setStandardOscillatorConnector(goal_move_left2_1,goal_move_left2_2,weight)
    setStandardOscillatorConnector(goal_move_left2_2,goal_move_left2_1,weight)
    setStandardOscillatorConnector(goal_move_right1_1,goal_move_right1_2,weight)
    setStandardOscillatorConnector(goal_move_right1_2,goal_move_right1_1,weight)
    setStandardOscillatorConnector(goal_move_right2_1,goal_move_right2_2,weight)
    setStandardOscillatorConnector(goal_move_right2_2,goal_move_right2_1,weight)
    setStandardOscillatorConnector(goal_pyramid_1,goal_pyramid_2,weight)
    setStandardOscillatorConnector(goal_pyramid_2,goal_pyramid_1,weight)


    #connect compound goals to second part of goals
    setStandardOscillatorConnector(goal_move_left1_1,goal_move_left2_1,synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_move_left1_2,goal_move_left2_1,synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_move_right1_1,goal_move_right2_1,synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_move_right1_2,goal_move_right2_1,synWeightCoeff*synWeight)

    #connections for modules
    setStandardOscillatorConnector(moduleTL_1,moduleTL_2,weight)
    setStandardOscillatorConnector(moduleTL_2,moduleTL_1,weight)
    setStandardOscillatorConnector(moduleTR_1,moduleTR_2,weight)
    setStandardOscillatorConnector(moduleTR_2,moduleTR_1,weight)
    setStandardOscillatorConnector(moduleBack_1,moduleBack_2,weight)
    setStandardOscillatorConnector(moduleBack_2,moduleBack_1,weight)
    setStandardOscillatorConnector(moduleForward_1,moduleForward_2,weight)
    setStandardOscillatorConnector(moduleForward_2,moduleForward_1,weight)

    #connections between modules and goals
    setStandardOscillatorConnector(goalTL_1,moduleTL_2,weight)
    setStandardOscillatorConnector(goalTL_2,moduleTL_1,weight)
    setStandardOscillatorConnector(goalTL_1,moduleTL_1,weight)
    setStandardOscillatorConnector(goalTL_2,moduleTL_2,weight)
    setStandardOscillatorConnector(goalTR_1,moduleTR_2,weight)
    setStandardOscillatorConnector(goalTR_2,moduleTR_1,weight)
    setStandardOscillatorConnector(goalTR_1,moduleTR_1,weight)
    setStandardOscillatorConnector(goalTR_2,moduleTR_2,weight)
    setStandardOscillatorConnector(goalBack_1,moduleBack_2,weight)
    setStandardOscillatorConnector(goalBack_2,moduleBack_1,weight)
    setStandardOscillatorConnector(goalBack_1,moduleBack_1,weight)
    setStandardOscillatorConnector(goalBack_2,moduleBack_2,weight)
    setStandardOscillatorConnector(goalForward_1,moduleForward_2,weight)
    setStandardOscillatorConnector(goalForward_2,moduleForward_1,weight)
    setStandardOscillatorConnector(goalForward_1,moduleForward_1,weight)
    setStandardOscillatorConnector(goalForward_2,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_left1_1,moduleTL_2,weight)
    setStandardOscillatorConnector(goal_move_left1_2,moduleTL_1,weight)
    setStandardOscillatorConnector(goal_move_left1_1,moduleTL_1,weight)
    setStandardOscillatorConnector(goal_move_left1_2,moduleTL_2,weight)
    setStandardOscillatorConnector(goal_move_left2_1,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_left2_2,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_left2_1,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_left2_2,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_right1_1,moduleTR_2,weight)
    setStandardOscillatorConnector(goal_move_right1_2,moduleTR_1,weight)
    setStandardOscillatorConnector(goal_move_right1_1,moduleTR_1,weight)
    setStandardOscillatorConnector(goal_move_right1_2,moduleTR_2,weight)
    setStandardOscillatorConnector(goal_move_right2_1,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_right2_2,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_right2_1,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_right2_2,moduleForward_2,weight)

    #connections between goal_pyramid (find the pyramid) and modules.
    #very weak and passive, since pyramid could be either left or right
    setStandardOscillatorConnector(goal_pyramid_1, moduleTR_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_1, moduleTR_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2, moduleTR_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2, moduleTR_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_1, moduleTL_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_1, moduleTL_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2, moduleTL_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2,moduleTL_2,synWeightCoeff*synWeight)

    #facts have stronger weights and more bias, excite the **_1  lead to spike
    synWeightCoeff = 0.75
    chrisWeight = 1.0
    setStandardOscillatorConnector(leftFactAPop,moduleTL_1,chrisWeight)
    setStandardOscillatorConnector(leftFactBPop,moduleTL_1,chrisWeight)
    setStandardOscillatorConnector(rightFactAPop,moduleTR_1,chrisWeight)
    setStandardOscillatorConnector(rightFactBPop,moduleTR_1,chrisWeight)

    #facts inhibit modules
    chrisInhib = -3.0
    setStandardOscillatorConnector(leftFactAPop,moduleTR_1,chrisInhib)
    setStandardOscillatorConnector(leftFactAPop,moduleTR_2,chrisInhib)
    setStandardOscillatorConnector(rightFactAPop,moduleTL_1,chrisInhib)
    setStandardOscillatorConnector(rightFactAPop,moduleTL_2,chrisInhib)

    #to and from actions
    setStandardOscillatorConnector(moduleTL_1, TL, synWeight)
    setStandardOscillatorConnector(moduleTL_2, TL, synWeight)
    setStandardOscillatorConnector(TL,moduleTL_1, inhibitW)
    setStandardOscillatorConnector(TL,moduleTL_2, inhibitW)
    setStandardOscillatorConnector(TL,goalTL_1, inhibitW)
    setStandardOscillatorConnector(TL,goalTL_2, inhibitW)

    setStandardOscillatorConnector(moduleTR_1, TR, synWeight)
    setStandardOscillatorConnector(moduleTR_2, TR, synWeight)
    setStandardOscillatorConnector(TR,moduleTR_1, inhibitW)
    setStandardOscillatorConnector(TR,moduleTR_2, inhibitW)
    setStandardOscillatorConnector(TR,goalTR_1, inhibitW)
    setStandardOscillatorConnector(TR,goalTR_2, inhibitW)

    setStandardOscillatorConnector(moduleBack_1, back, synWeight)
    setStandardOscillatorConnector(moduleBack_2, back, synWeight)
    setStandardOscillatorConnector(back,moduleBack_1, inhibitW)
    setStandardOscillatorConnector(back,moduleBack_2, inhibitW)
    setStandardOscillatorConnector(back,goalBack_1, inhibitW)
    setStandardOscillatorConnector(back,goalBack_2, inhibitW)

    setStandardOscillatorConnector(moduleForward_1, forward, synWeight)
    setStandardOscillatorConnector(moduleForward_2, forward, synWeight)
    setStandardOscillatorConnector(forward,moduleForward_1, inhibitW)
    setStandardOscillatorConnector(forward,moduleForward_2, inhibitW)
    setStandardOscillatorConnector(forward,goalForward_1, inhibitW)
    setStandardOscillatorConnector(forward,goalForward_2, inhibitW)

    #action left and forward for move left
    synWeightCoeff=0.75
    setStandardOscillatorConnector(TL, goal_move_left2_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TL, goal_move_left2_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TL, goal_move_left1_1, inhibitW)
    setStandardOscillatorConnector(TL, goal_move_left1_2, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_left2_1, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_left2_2, inhibitW)

    setStandardOscillatorConnector(TR, goal_move_right2_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TR, goal_move_right2_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TR, goal_move_right1_1, inhibitW)
    setStandardOscillatorConnector(TR, goal_move_right1_2, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_right2_1, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_right2_2, inhibitW)

    #actions turn off facts too (thanks Ian)
    setStandardOscillatorConnector(TR, rightFactAPop, inhibitW)
    setStandardOscillatorConnector(TR, rightFactBPop, inhibitW)
    setStandardOscillatorConnector(TL, leftFactAPop, inhibitW)
    setStandardOscillatorConnector(TL, leftFactBPop, inhibitW)

    #actions turn off pyramid goals sJA
    setStandardOscillatorConnector(TL, goal_pyramid_1, inhibitW)
    setStandardOscillatorConnector(TL, goal_pyramid_2, inhibitW)
    #eja
    setStandardOscillatorConnector(TR, goal_pyramid_1, inhibitW)
    setStandardOscillatorConnector(TR, goal_pyramid_2, inhibitW)

#-------------Print out neuron by neuron the firing times
def print_spikes(of, label, pop):
	print "\n\n%s\n" % label
	of.write("\n%s\n" % label)
	outDat = pop.get_data()
	for seg in outDat.segments:
		print(seg.spiketrains)
		of.write("\n%s\n" % str(seg.spiketrains))

def printReceptorFirings(receptorPop):
   totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
   for neuronNum in range (0,totalNeurons-1):
       if (neuronNum < totalNeurons):
           outAssembly = Assembly(receptorPop[neuronNum,(neuronNum+1)])
       else: 
           outAssembly = Assembly(receptorPop[neuronNum,0])
       outDat = outAssembly.get_data()
       seg = outDat.segments[0]
       spikeTrains = seg.spiketrains
       times = spikeTrains[0].times
       if len(times) > 0:
           print neuronNum, len(times),
           for time in times:
               base = time.base
               print base,
           print " "

def printReceptors():
    print "3x3 On"
    printReceptorFirings(retina3x3OnPop)
    print "3x3 Off"
#    printReceptorFirings(retina3x3OffPop)
    print "6x6 On"
    printReceptorFirings(retina6x6OnPop)
    print "6x6 Off"
    printReceptorFirings(retina6x6OffPop)
    print "9x9 On"
#    printReceptorFirings(retina9x9OnPop)
    print "9x9 Off"
    printReceptorFirings(retina9x9OffPop)

def printV1():
    print "HEdgeBottom"
    printReceptorFirings(v1HEdgeBottomPop)
    print "SEdgeRight"
    printReceptorFirings(v1SEdgeRightPop)
    print "BEdgeLeft"
    printReceptorFirings(v1BEdgeLeftPop)
    print "AndAngle"
    printReceptorFirings(v1AndAnglePop)
    print "LessThanAngle"
    printReceptorFirings(v1LessThanAnglePop)
    print "GreaterThanAngle"
    printReceptorFirings(v1GreaterThanAnglePop)

#------Image Stuff
def openImageFile():
    try: 
#pyramid        im = Image.open("ed_3screenshot.jpg")
#stalactite        im = Image.open("ed_screenshot826.jpg") #stalactite
        im = Image.open("edR_screenshot826.jpg") #stalactite right
    except:
        print "fail to open image"

    im = im.crop((0,0,600,600))
    #print im.size

    #shrink it
    size = (INPUT_NEURONS_WIDTH,INPUT_NEURONS_HEIGHT)
    im.thumbnail(size)

    #get the pixels
    pixels = im.load()

    ##convert rgb to grey scale
    inputMatrix = numpy.zeros((50,50))
    for inpRow in range (0,50):
        for inpCol in range (0,50):
            rgb = pixels[inpCol,inpRow]
            for i in range (0,3):
                inputMatrix[inpCol,inpRow] += rgb[i]
                inputMatrix[inpCol,inpRow]/=3
        
    ##set result image array
    resultImage = numpy.zeros((50,50))
    THRESHOLD = 50
    resultImageArray = []
    for outRow in range (0,50):
        for outColumn in range (0,50):
            if inputMatrix[outColumn,outRow] < THRESHOLD :
                #print outRow, outColumn
                pixelOn = (outRow*INPUT_NEURONS_WIDTH)+outColumn
                resultImageArray = resultImageArray + [pixelOn]

    return resultImageArray

#stimulus
def setInputs():
        pulse0 =DCSource(amplitude=0.0055,start=0.0,stop=10.0)
	ML_Assembly = Assembly(goal_move_left1_1[0,1,2,3,4])
	pulse0.inject_into(ML_Assembly)
	pulse1 =DCSource(amplitude=0.0055,start=100.0,stop=110.0)
	TP_Assembly = Assembly(goal_pyramid_1[0,1,2,3,4])
        pulse1.inject_into(TP_Assembly)
	pulse2 =DCSource(amplitude=0.0055,start=200.0,stop=210.0)
	back_Assembly = Assembly(goalBack_1[0,1,2,3,4])
	pulse2.inject_into(back_Assembly)

def setInputs2():
        pulse0 =DCSource(amplitude=0.0055,start=0.0,stop=10.0)
	TR_Assembly = Assembly(goalTR_1[0,1,2,3,4])
	pulse0.inject_into(TR_Assembly)
	pulse1 =DCSource(amplitude=0.0055,start=100.0,stop=110.0)
	TL_Assembly = Assembly(goalTL_1[0,1,2,3,4])
        pulse1.inject_into(TL_Assembly)
	pulse2 =DCSource(amplitude=0.0055,start=200.0,stop=210.0)
	forward_Assembly = Assembly(goalForward_1[0,1,2,3,4])
	pulse2.inject_into(forward_Assembly)
	pulse3 =DCSource(amplitude=0.0055,start=300.0,stop=310.0)
	back_Assembly = Assembly(goalBack_1[0,1,2,3,4])
	pulse3.inject_into(back_Assembly)
	pulse4 =DCSource(amplitude=0.0055,start=400.0,stop=410.0)
	move_left1_Assembly = Assembly(goal_move_left1_1[0,1,2,3,4])
	pulse4.inject_into(move_left1_Assembly)
	pulse5 = DCSource(amplitude=0.0055, start=500.0, stop=510.0)
	move_right1_Assembly = Assembly(goal_move_right1_1[0,1,2,3,4])
	pulse5.inject_into(move_right1_Assembly)
	pulse6 = DCSource(amplitude=0.0055, start=600.0, stop=610.0)
	move_left1_Assembly = Assembly(goal_move_left1_1[0,1,2,3,4])
	pulse6.inject_into(move_left1_Assembly)
	pulse7 = DCSource(amplitude=0.0055, start=700.0, stop=710.0)
	pulse7.inject_into(TR_Assembly)
	pulse8 = DCSource(amplitude=0.0055, start=800.0, stop=810.0)
	pulse8.inject_into(TL_Assembly)
	pulse9 =DCSource(amplitude=0.0055,start=900.0,stop=910.0)
	forward_Assembly = Assembly(goalForward_1[0,1,2,3,4])
	pulse9.inject_into(forward_Assembly)
	pulse10 =DCSource(amplitude=0.0055, start=1000.0,stop=1010.0)
	pulse10.inject_into(TR_Assembly)
	pulse11 =DCSource(amplitude=0.0055,start=1100.0,stop=1110.0)
	pulse11.inject_into(TL_Assembly)
	pulse12 =DCSource(amplitude=0.0055,start=1200.0,stop=1210.0)
	TP_Assembly = Assembly(goal_pyramid_1[0,1,2,3,4])
	pulse12.inject_into(TP_Assembly)
	pulse13 =DCSource(amplitude=0.0055,start=1300.0,stop=1310.0)
	pulse13.inject_into(TP_Assembly)
	pulse14 =DCSource(amplitude=0.0055,start=1400.0,stop=1410.0)
	pulse14.inject_into(TR_Assembly)
	pulse15 =DCSource(amplitude=0.0055,start=1500.0,stop=1510.0)
	pulse15.inject_into(TL_Assembly)

        
def runNoCallBack():
    #set up clamped input from image
    pulseVis = DCSource(amplitude=0.0038, start=0.0, stop=SIM_LENGTH)
    inputAssembly = Assembly(inpPop[imagePixels])
    pulseVis.inject_into(inputAssembly)

    #pulseGoal = DCSource(amplitude=0.0038, start=0.0, stop=10)
    #goalInputAssembly = Assembly(goal_move_left1_1[0,1,2,3,4])
    #goalInputAssembly = Assembly(goal_pyramid_1[0,1,2,3,4])
    #pulseGoal.inject_into(goalInputAssembly)

    run(SIM_LENGTH)

#------------Main Body---------------
simulator_name = get_script_args(1)[0]  
exec("from pyNN.%s import *" % simulator_name)

setup(timestep=DELAY,min_delay=DELAY,max_delay=DELAY,debug=0)

#----------------create neurons
#input neurons
cInputNeurons = INPUT_NEURONS_HEIGHT * INPUT_NEURONS_WIDTH 

#default a .02, b .2, c -65, d 2.0, i_offset 0
inp_cell_params = {'a' : 0.0, 'b' :0.2, 'c' : -65, 'd' : 0.0}
inpPop = Population(cInputNeurons,Izhikevich(**inp_cell_params))

inpPop.initialize(v=-58.0)

#create retinal neurons off centre different than on centre
retina_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0}
retina3x3OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina6x6OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina9x9OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))

retina_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0,'i_offset':1.0}
retina3x3OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina6x6OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina9x9OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))

#create V1 neurons
V1_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0}
v1HEdgeBottomPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1SEdgeRightPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1BEdgeLeftPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1AndAnglePop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1LessThanAnglePop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1GreaterThanAnglePop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1HEdgeTopPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1SEdgeRightPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1BEdgeLeftPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1OrAnglePop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))

#create object recognition neurons
obj_cell_params = {'a' : 0.02,'i_offset' : 2}
pyramidPop = Population(cInputNeurons,Izhikevich(**obj_cell_params))
stalactitePop = Population(cInputNeurons,Izhikevich(**obj_cell_params))

#create fact neurons (for vision)
fact_cell_params = {'a': 0.05, 'd': 0,'i_offset' : 2}
leftFactAPop = Population(5,Izhikevich(**fact_cell_params))
leftFactBPop = Population(5,Izhikevich(**fact_cell_params))
rightFactAPop = Population(5,Izhikevich(**fact_cell_params))
rightFactBPop = Population(5,Izhikevich(**fact_cell_params))
pyramidFactAPop = Population(5,Izhikevich(**fact_cell_params))
pyramidFactBPop = Population(5,Izhikevich(**fact_cell_params))
stalactiteFactAPop = Population(5,Izhikevich(**fact_cell_params))
stalactiteFactBPop = Population(5,Izhikevich(**fact_cell_params))
    

##create plan neurons
cell_params = {'a' : 0.05}
goalTL_1 = Population(5,Izhikevich(**cell_params))
goalTL_2 = Population(5,Izhikevich(**cell_params))

goalTR_1 = Population(5, Izhikevich(**cell_params))
goalTR_2 = Population(5, Izhikevich(**cell_params))

goalBack_1 = Population(5,Izhikevich(**cell_params))
goalBack_2 = Population(5,Izhikevich(**cell_params))

goalForward_1 = Population(5, Izhikevich(**cell_params))
goalForward_2 = Population(5, Izhikevich(**cell_params))

goal_move_left1_1 = Population(5, Izhikevich(**cell_params))
goal_move_left1_2 = Population(5, Izhikevich(**cell_params))

goal_move_left2_1 = Population(5, Izhikevich(**cell_params))
goal_move_left2_2 = Population(5, Izhikevich(**cell_params))

goal_move_right1_1 = Population(5, Izhikevich(**cell_params))
goal_move_right1_2 = Population(5, Izhikevich(**cell_params))

goal_move_right2_1 = Population(5, Izhikevich(**cell_params))
goal_move_right2_2 = Population(5, Izhikevich(**cell_params))


goal_pyramid_1 = Population(5, Izhikevich(**cell_params))
goal_pyramid_2 = Population(5, Izhikevich(**cell_params))


#FACTS	see facts in vision 

#modules 
moduleTL_1 = Population(5,Izhikevich(**cell_params))
moduleTL_2 = Population(5,Izhikevich(**cell_params))
moduleTR_1 = Population(5, Izhikevich(**cell_params))
moduleTR_2 = Population(5, Izhikevich(**cell_params))
moduleBack_1 = Population(5,Izhikevich(**cell_params))
moduleBack_2 = Population(5,Izhikevich(**cell_params))
moduleForward_1 = Population(5, Izhikevich(**cell_params))
moduleForward_2 = Population(5, Izhikevich(**cell_params))


#output/actions
TL = Population(1, Izhikevich(**cell_params))
TR = Population(1, Izhikevich(**cell_params))
back = Population(1, Izhikevich(**cell_params))
forward = Population(1, Izhikevich(**cell_params))


#----------------setup input to retina connections
synapseArray = []
#3x3On
for row in range (1,49):
    for col in range (1,49):
        newSynapses = set3x3OnConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina3x3OnPop,FromListConnector(synapseArray),StaticSynapse())

#3x3Off
#synapseArray = []
#for row in range (1,49):
#    for col in range (1,49):
#        newSynapses = set3x3OffConnections(col,row)
#        synapseArray = synapseArray + newSynapses
#Projection(inpPop,retina3x3OffPop,FromListConnector(synapseArray),StaticSynapse())

#6x6On
synapseArray = []
for row in range (2,47):
    for col in range (2,47):
        newSynapses = set6x6OnConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina6x6OnPop,FromListConnector(synapseArray),StaticSynapse())

#6x6Off
synapseArray = []
for row in range (2,47):
    for col in range (2,47):
        newSynapses = set6x6OffConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina6x6OffPop,FromListConnector(synapseArray),StaticSynapse())

#9x9Off
synapseArray = []
for row in range (3,45):
    for col in range (3,45):
        newSynapses = set9x9OffConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina9x9OffPop,FromListConnector(synapseArray),StaticSynapse())

#9x9On
#synapseArray = []
#for row in range (3,45):
#    for col in range (3,45):
#        newSynapses = set9x9OnConnections(col,row)
#        synapseArray = synapseArray + newSynapses
#Projection(inpPop,retina9x9OnPop,FromListConnector(synapseArray),StaticSynapse())

#-------------------setup retina to V1 connections---
#Hedge
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (2,INPUT_NEURONS_HEIGHT-3):
        newSynapses = set6x6OnToHEdgeBottomConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina6x6OnPop,v1HEdgeBottomPop,FromListConnector(synapseArray),StaticSynapse())
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (2,INPUT_NEURONS_HEIGHT-3):
        newSynapses = set6x6OffToHEdgeBottomConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina6x6OffPop,v1HEdgeBottomPop,FromListConnector(synapseArray),StaticSynapse())

#SEdge Bottom Connections
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set6x6OnToSEdgeLeftConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina6x6OnPop,v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())
Projection(retina6x6OffPop,v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

#leftBEdges
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set6x6OnToBEdgeRightConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina6x6OnPop,v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())
Projection(retina6x6OffPop,v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())

#andAngle
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set3x3OnToAndAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina3x3OnPop,v1AndAnglePop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set9x9OffToAndAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1AndAnglePop,FromListConnector(synapseArray),StaticSynapse())

#Less Than Angle
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set3x3OnToLessThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina3x3OnPop,v1LessThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set9x9OffToLessThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1LessThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

#Greater Than Angle
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set3x3OnToGreaterThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina3x3OnPop,v1GreaterThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set9x9OffToGreaterThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina9x9OffPop,v1GreaterThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

#topHEdges
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (0,INPUT_NEURONS_HEIGHT):
        newSynapses = set6x6OffToHEdgeTopConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina6x6OffPop,v1HEdgeTopPop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (0,INPUT_NEURONS_HEIGHT):
        newSynapses = set9x9OffToHEdgeTopConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1HEdgeTopPop,FromListConnector(synapseArray),StaticSynapse())

#right SEdges
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (0,INPUT_NEURONS_HEIGHT):
        newSynapses = set9x9OffToSEdgeRightConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

#left BEdges
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (0,INPUT_NEURONS_HEIGHT):
        newSynapses = set9x9OffToBEdgeLeftConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())

#or Angle
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set3x3OnToOrAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina3x3OnPop,v1OrAnglePop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (0,INPUT_NEURONS_HEIGHT):
        newSynapses = set9x9OffToOrAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1OrAnglePop,FromListConnector(synapseArray),StaticSynapse())

#-------------------setup Object Recognion and  V1 to ObjRec connections---
print "V1 connections"
#setup pyramid CAs
synapseArray = []
for cCA in range (0,25):
    newSynapses = connectCA(cCA)
    synapseArray = synapseArray + newSynapses
Projection(pyramidPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#HEdge to Pyramid
synapseArray = connectHEdgeBottomToPyramid()
Projection(v1HEdgeBottomPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#SEdge to Pyramid
synapseArray = connectSEdgeLeftToPyramid()
Projection(v1SEdgeRightPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#BEdge to Pyramid
synapseArray = connectBEdgeRightToPyramid()
Projection(v1BEdgeLeftPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#And to Pyramid
synapseArray = connectAndAngleToPyramid()
Projection(v1AndAnglePop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#Less Than Angle to Pyramid
synapseArray = connectLessThanAngleToPyramid()
Projection(v1LessThanAnglePop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#Greater Than Angle to Pyramid
synapseArray = connectGreaterThanAngleToPyramid()
Projection(v1GreaterThanAnglePop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#---------------setup stalactite CAs
synapseArray = []
for cCA in range (0,25):
    newSynapses = connectCA(cCA)
    synapseArray = synapseArray + newSynapses
Projection(stalactitePop,stalactitePop,FromListConnector(synapseArray),StaticSynapse())

#HEdge to Stalactite
synapseArray = connectHEdgeTopToStalactite()
Projection(v1HEdgeTopPop,stalactitePop,FromListConnector(synapseArray),StaticSynapse())

#sEdge to Stalactite
synapseArray = connectSEdgeRightToStalactite()
Projection(v1SEdgeRightPop,stalactitePop,FromListConnector(synapseArray),StaticSynapse())

#bEdge to Stalactite
synapseArray = connectBEdgeLeftToStalactite()
Projection(v1BEdgeLeftPop,stalactitePop,FromListConnector(synapseArray),StaticSynapse())

#orAngle to Stalactite
synapseArray = connectOrAngleToStalactite()
Projection(v1OrAnglePop,stalactitePop,FromListConnector(synapseArray),StaticSynapse())

#setup fact connections
synWeight = 5.0
connect(leftFactAPop, leftFactBPop, weight=synWeight, delay=DELAY)
connect(leftFactBPop, leftFactAPop, weight=synWeight, delay=DELAY)
connect(rightFactAPop, rightFactBPop, weight=synWeight, delay=DELAY)
connect(rightFactBPop, rightFactAPop, weight=synWeight, delay=DELAY)
connect(pyramidFactAPop, pyramidFactBPop, weight=synWeight, delay=DELAY)
connect(pyramidFactBPop, pyramidFactAPop, weight=synWeight, delay=DELAY)
connect(stalactiteFactAPop, stalactiteFactBPop, weight=synWeight, delay=DELAY)
connect(stalactiteFactBPop, stalactiteFactAPop, weight=synWeight, delay=DELAY)


#-------connect objects to facts
synapseArray = leftObjectTurnsLeftFactOn()
Projection(pyramidPop,leftFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(stalactitePop,leftFactAPop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = rightObjectTurnsLeftFactOn()
Projection(pyramidPop,rightFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(stalactitePop,rightFactAPop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = objectTurnsObjectFactOn()
Projection(pyramidPop,pyramidFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(stalactitePop,stalactiteFactAPop,FromListConnector(synapseArray),StaticSynapse())


print "Plan Connections"
setPlanConnections()

#turn on the inputs
setInputs()

#-------------------setup recording
inpPop.record(['spikes','v'])
retina3x3OnPop.record(['spikes','v'])
#retina3x3OffPop.record(['spikes','v'])
retina6x6OnPop.record(['spikes','v'])
retina6x6OffPop.record(['spikes','v'])
#retina9x9OnPop.record(['spikes','v'])
retina9x9OffPop.record(['spikes','v'])
v1HEdgeBottomPop.record(['spikes','v'])
v1SEdgeRightPop.record(['spikes','v'])
v1BEdgeLeftPop.record(['spikes','v'])
v1AndAnglePop.record(['spikes','v'])
v1LessThanAnglePop.record(['spikes','v'])
v1GreaterThanAnglePop.record(['spikes','v'])
pyramidPop.record(['spikes','v'])
stalactitePop.record(['spikes','v'])
leftFactAPop.record(['spikes','v'])
rightFactAPop.record(['spikes','v'])
pyramidFactAPop.record(['spikes','v'])
stalactiteFactAPop.record(['spikes','v'])
back.record('spikes')
forward.record('spikes')
TR.record('spikes')
TL.record('spikes')

#comment - open file
of = open(output_filename, 'w')

#print inpPop[0].get_parameters()

#open image file
imagePixels = openImageFile()
#print imagePixels

runNoCallBack()

#--------------print results
print "left fact"
outAss = Assembly(leftFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)
#    print(seg.analogsignalarrays)
#printReceptors()
#printV1()

print "right fact"
outAss = Assembly(rightFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)
#    print(seg.analogsignalarrays)
#printReceptors()
#printV1()

print "pyramid fact"
outAss = Assembly(pyramidFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)

print "stalactite fact"
outAss = Assembly(stalactiteFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)

print "pyramid"
outAss = Assembly(pyramidPop[821,520])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)
print "stalactite"
outAss = Assembly(stalactitePop[821,520])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)

print_spikes(of,"Pop Back", back)
print_spikes(of,"Pop Forward", forward)
print_spikes(of,"Pop Turn Right", TR)
print_spikes(of,"Pop Turn Left", TL)



end()

