"""
Connections from the Retina to V1
The hEdgeBottom, sEdgeLeft, bEdgeRight, andAngle, lessThanAngle,
greaterThanAngle, HedgeTop, sEdgeRight, bEdgeLeft and orAngle
Connection Functions
"""

INPUT_NEURONS_HEIGHT = 50
INPUT_NEURONS_WIDTH = 50

DELAY = 1.0

#----------setup connections 6x6 on to HEdgeBottom (bottom of a solid)
def set6x6OnToHEdgeBottomConnections(Col,Row):
    weight = 3.0
    #don't input off top
    if (Row == 0): return []
    #don't output off bottom
    if (Row == INPUT_NEURONS_HEIGHT): return []
    connector = []
    toNeuron = (Row)*50+Col
    startAddCol = Col-2
    if (startAddCol < 0): startAddCol = 0
    endAddCol = Col+3
    if (endAddCol > INPUT_NEURONS_WIDTH): endAddCol = INPUT_NEURONS_WIDTH
    for addCol in range (startAddCol,endAddCol):
        fromNeuron = ((Row-1)*50)+addCol
        newConnector = [(fromNeuron, toNeuron, weight, DELAY)]
        connector = connector + newConnector
    return connector

#----------setup connections 6x6off to HEdgeBottom (bottom of a solid)
def set6x6OffToHEdgeBottomConnections(Col,Row):
    weight = 3.0
    #don't input off top
    if (Row == 0): return []
    #don't output off bottom
    if (Row == INPUT_NEURONS_HEIGHT): return []
    connector = []
    toNeuron = (Row)*50+Col
    startAddCol = Col-2
    if (startAddCol < 0): startAddCol = 0
    endAddCol = Col+3
    if (endAddCol > INPUT_NEURONS_WIDTH): endAddCol = INPUT_NEURONS_WIDTH
    for addCol in range (startAddCol,endAddCol):
        fromNeuron = ((Row+1)*50)+addCol
        newConnector = [(fromNeuron, toNeuron, weight, DELAY)]
        connector = connector + newConnector
    return connector

#----------setup connections 6x6 to SEdgeLeft (left of a solid)
#connections from self one to right then 3 above 2 below sloping
def set6x6OnToSEdgeLeftConnections(Col,Row):
    if (Row <3): return []
    if (Row > INPUT_NEURONS_HEIGHT -3): return []
    if (Col < 2): return []
    if (Col > INPUT_NEURONS_WIDTH -4): return []

    weight = 4.0
    inhibWeight = -4.0 # inhibit right sEdges well to the rigft
    toNeuron = (Row)*50+Col
    row1Off = (Row-3)*INPUT_NEURONS_WIDTH + Col+3
    row1Left = (Row-3)*INPUT_NEURONS_WIDTH + Col+2
    row1Right = (Row-3)*INPUT_NEURONS_WIDTH + Col+1
    row2Off = (Row-2)*INPUT_NEURONS_WIDTH + Col+2
    row2Left = (Row-2)*INPUT_NEURONS_WIDTH + Col+1
    row2Right = (Row-2)*INPUT_NEURONS_WIDTH + Col
    row3Off = (Row-1)*INPUT_NEURONS_WIDTH + Col+2
    row3Left = (Row-1)*INPUT_NEURONS_WIDTH + Col+1
    row3Right = (Row-1)*INPUT_NEURONS_WIDTH + Col
    ownRowOff = Row*INPUT_NEURONS_WIDTH + Col+1
    ownRowLeft = toNeuron
    ownRowRight = toNeuron - 1
    row5Off = (Row+1)*INPUT_NEURONS_WIDTH + Col+1
    row5Left = (Row+1)*INPUT_NEURONS_WIDTH + Col
    row5Right = (Row+1)*INPUT_NEURONS_WIDTH + Col-1
    row6Off = (Row+2)*INPUT_NEURONS_WIDTH + Col
    row6Left = (Row+2)*INPUT_NEURONS_WIDTH + Col-1
    row6Right = (Row+2)*INPUT_NEURONS_WIDTH + Col-2
    connector = [
        (row1Off, toNeuron, inhibWeight, DELAY),
        (row1Left, toNeuron, weight, DELAY),
        (row1Right, toNeuron, weight, DELAY),
        (row2Off, toNeuron, inhibWeight, DELAY),
        (row2Left, toNeuron, weight, DELAY),
        (row2Right, toNeuron, weight, DELAY),
        (row3Off, toNeuron, inhibWeight, DELAY),
        (row3Left, toNeuron, weight, DELAY),
        (row3Right, toNeuron, weight, DELAY),        
        (ownRowOff, toNeuron, inhibWeight, DELAY),
        (ownRowLeft, toNeuron, weight, DELAY),
        (ownRowRight, toNeuron, weight, DELAY),
        (row5Off, toNeuron, inhibWeight, DELAY),
        (row5Left, toNeuron, weight, DELAY),
        (row5Right, toNeuron, weight, DELAY),        
        (row6Off, toNeuron, inhibWeight, DELAY),
        (row6Left, toNeuron, weight, DELAY),
        (row6Right, toNeuron, weight, DELAY)]
    return connector

#----------setup connections 6x6 on to BEdgeRight (right of a solid)
#connections from self one to left then 3 above 2 below sloping
def set6x6OnToBEdgeRightConnections(Col,Row):
    if (Row < 3): return []
    if (Row > INPUT_NEURONS_HEIGHT -3): return []
    if (Col < 3): return []
    if (Col > INPUT_NEURONS_WIDTH -3): return []

    weight = 4.0
    inhibWeight = -4.0 # inhibit things well to the left
    toNeuron = (Row)*50+Col
    row1Off = (Row-3)*INPUT_NEURONS_WIDTH + Col-3
    row1Left = (Row-3)*INPUT_NEURONS_WIDTH + Col-2
    row1Right = (Row-3)*INPUT_NEURONS_WIDTH + Col-1
    row2Off = (Row-2)*INPUT_NEURONS_WIDTH + Col-2
    row2Left = (Row-2)*INPUT_NEURONS_WIDTH + Col-1
    row2Right = (Row-2)*INPUT_NEURONS_WIDTH + Col
    row3Off = (Row-1)*INPUT_NEURONS_WIDTH + Col-2
    row3Left = (Row-1)*INPUT_NEURONS_WIDTH + Col-1
    row3Right = (Row-1)*INPUT_NEURONS_WIDTH + Col
    ownRowOff = Row*INPUT_NEURONS_WIDTH + Col-1
    ownRowLeft = toNeuron
    ownRowRight = toNeuron + 1
    row5Off = (Row+1)*INPUT_NEURONS_WIDTH + Col-1
    row5Left = (Row+1)*INPUT_NEURONS_WIDTH + Col
    row5Right = (Row+1)*INPUT_NEURONS_WIDTH + Col+1
    row6Off = (Row+2)*INPUT_NEURONS_WIDTH + Col
    row6Left = (Row+2)*INPUT_NEURONS_WIDTH + Col+1
    row6Right = (Row+2)*INPUT_NEURONS_WIDTH + Col+2
    connector = [
        (row1Off, toNeuron, inhibWeight, DELAY),
        (row1Left, toNeuron, weight, DELAY),
        (row1Right, toNeuron, weight, DELAY),
        (row2Off, toNeuron, inhibWeight, DELAY),
        (row2Left, toNeuron, weight, DELAY),
        (row2Right, toNeuron, weight, DELAY),
        (row3Off, toNeuron, inhibWeight, DELAY),
        (row3Left, toNeuron, weight, DELAY),
        (row3Right, toNeuron, weight, DELAY),        
        (ownRowOff, toNeuron, inhibWeight, DELAY),
        (ownRowLeft, toNeuron, weight, DELAY),
        (ownRowRight, toNeuron, weight, DELAY),
        (row5Off, toNeuron, inhibWeight, DELAY),
        (row5Left, toNeuron, weight, DELAY),
        (row5Right, toNeuron, weight, DELAY),        
        (row6Off, toNeuron, inhibWeight, DELAY),
        (row6Left, toNeuron, weight, DELAY),
        (row6Right, toNeuron, weight, DELAY)]
    return connector


#----------setup connections 3x3 on to And Angle 
#connections from self to above two rows and on either side for 6 in total
def set3x3OnToAndAngleConnections(Col,Row):
    if (Row > INPUT_NEURONS_HEIGHT -3): return []
    if (Col < 1): return []
    if (Col > INPUT_NEURONS_WIDTH-2): return []

    weight = 7.0

    toNeuron = (Row*50)+Col
    connector = []
    for addRow in range (Row+1,Row+3):
      for addCol in range (Col-1,Col+2):
        fromNeuron = ((addRow)*50)+addCol
        newConnector = [(fromNeuron, toNeuron, weight, DELAY)]
        connector = connector + newConnector
    return connector

#connect 9x9off inhib from  two to the left and right (5)
def set9x9OffToAndAngleConnections(Col,Row):
    if (Col < 2): return []
    if (Col > INPUT_NEURONS_WIDTH-3): return []

    inhibWeight = -4.0

    toNeuron = (Row*50)+Col
    fromInhibNeuron = (Row*50)+Col-2
    connector = [
        (fromInhibNeuron, toNeuron, inhibWeight, DELAY),
        (fromInhibNeuron+1, toNeuron, inhibWeight, DELAY),
        (fromInhibNeuron+2, toNeuron, inhibWeight, DELAY),
        (fromInhibNeuron+3, toNeuron, inhibWeight, DELAY),
        (fromInhibNeuron+4, toNeuron, inhibWeight, DELAY)]

    return connector

#----------setup connections 3x3 on to Less Than Angle 
#connections from self and from right two cols and row on either side to self 7
def set3x3OnToLessThanAngleConnections(Col,Row):
    if (Row < 1): return []
    if (Row > INPUT_NEURONS_HEIGHT-2): return []
    if (Col > INPUT_NEURONS_WIDTH-3): return []

    weight = 4.0

    toNeuron = (Row*50)+Col
    fromNeuron = toNeuron
    connector =  [(fromNeuron,toNeuron,weight,DELAY)]
    for addRow in range (Row-1,Row+2):
      for addCol in range (Col+1,Col+3):
        fromNeuron = ((addRow)*50)+addCol
        newConnector = [(fromNeuron, toNeuron, weight, DELAY)]
        connector = connector + newConnector
    return connector

#connect 9x9off excite self inhib from  two to the left for 3 rows around (9)
def set9x9OffToLessThanAngleConnections(Col,Row):
    if (Row < 1): return []
    if (Row > INPUT_NEURONS_HEIGHT-2): return []
    if (Col < 2): return []

    weight = 5.0
    inhibWeight = -3.0

    toNeuron = (Row*50)+Col
    connector = []
    for addRow in range (Row-1,Row+2):
        fromNeuron = ((addRow)*50)+Col
        newConnectors = [(fromNeuron, toNeuron, weight, DELAY),
                        (fromNeuron-1, toNeuron, inhibWeight, DELAY),
                        (fromNeuron-2, toNeuron, inhibWeight, DELAY)]
        connector = connector + newConnectors
    return connector




#----------setup connections 3x3 on to And Angle 
#conns from self and from right left cols and row on either side to self 7
def set3x3OnToGreaterThanAngleConnections(Col,Row):
    if (Row < 1): return []
    if (Row > INPUT_NEURONS_HEIGHT-2): return []
    if (Col < 1): return []
    if (Col > INPUT_NEURONS_WIDTH-1): return []

    weight = 4.0

    toNeuron = (Row*50)+Col
    fromNeuron = toNeuron+1
    connector =  [(fromNeuron,toNeuron,weight,DELAY)]
    for addRow in range (Row-1,Row+2):
      for addCol in range (Col-1,Col+1):
        fromNeuron = ((addRow)*50)+addCol
        newConnector = [(fromNeuron, toNeuron, weight, DELAY)]
        connector = connector + newConnector
    return connector

#connect 9x9off excite from left inhib from self and right for 3 rows around(9)
def set9x9OffToGreaterThanAngleConnections(Col,Row):
    if (Row < 1): return []
    if (Row > INPUT_NEURONS_HEIGHT-2): return []
    if (Col < 2): return []
#    if (Col > INPUT_NEURONS_WIDTH-2): return []

    weight = 5.0
    inhibWeight = -1.0

    toNeuron = (Row*50)+Col
    connector = []
    for addRow in range (Row-1,Row+2):
        fromNeuron = ((addRow)*50)+Col
        newConnectors = [(fromNeuron-2, toNeuron, weight, DELAY),
                        (fromNeuron-1, toNeuron, inhibWeight, DELAY),
                        (fromNeuron, toNeuron, inhibWeight, DELAY)]
        connector = connector + newConnectors
    return connector


#----------setup connections 6x6off to HEdgeTop (top of a solid)
#--one above and 2 on either side for 5
def set6x6OffToHEdgeTopConnections(Col,Row):
    weight = 10.0
    #don't output off bottom
    if (Row == INPUT_NEURONS_HEIGHT): return []
    connector = []
    toNeuron = (Row)*50+Col
    startAddCol = Col-2
    if (startAddCol < 0): startAddCol = 0
    endAddCol = Col+3
    if (endAddCol > INPUT_NEURONS_WIDTH): endAddCol = INPUT_NEURONS_WIDTH
    for addCol in range (startAddCol,endAddCol):
        fromNeuron = ((Row+1)*50)+addCol
        newConnector = [(fromNeuron, toNeuron, weight, DELAY)]
        connector = connector + newConnector
    return connector

#----------setup connections 9x9 off to HEdgeTop (top of a solid)
#---positiive to same space, inhibit below
def set9x9OffToHEdgeTopConnections(Col,Row):
    weight = 5.0
    inhibWeight = -10
    #don't output off bottom
    if (Row == INPUT_NEURONS_HEIGHT): return []
    connector = []
    toNeuron = (Row)*50+Col
    fromNeuron = (Row*50)+Col
    connector = [(fromNeuron, toNeuron, weight, DELAY)]
    fromNeuron = ((Row+1)*50)+Col
    newConnector = [(fromNeuron, toNeuron, inhibWeight, DELAY)]
    connector = connector + newConnector
    return connector

#----------setup connections 9x9 off to SEdgeRight (right of a solid)
#---positive right, down, and 2rightoneup (3). Inhibit self.
def set9x9OffToSEdgeRightConnections(Col,Row):
    weight = 7.0
    inhibWeight = -10
    #don't input off top
    if (Row == 0): return []
    #don't output off bottom
    if (Row == INPUT_NEURONS_HEIGHT-1): return []
    #don't output off bottom
    if (Col >= INPUT_NEURONS_WIDTH -2): return []
    connector = []
    toNeuron = (Row)*50+Col
    fromNeuron = (Row*50)+Col
    connector = [(fromNeuron, toNeuron, inhibWeight, DELAY)]
    fromNeuron = (Row*50)+Col-1  #right
    connector = connector+ [(fromNeuron, toNeuron, weight, DELAY)]
    fromNeuron = ((Row-1)*50)+Col #below
    connector = connector+ [(fromNeuron, toNeuron, weight, DELAY)]
    fromNeuron = ((Row+1)*50)+Col-2 #above two right
    connector = connector + [(fromNeuron, toNeuron, weight, DELAY)]
    return connector

#----------setup connections 9x9 off to BEdgeLeft (left of a solid)
#---positive left, and down (2). Inhibit self.
def set9x9OffToBEdgeLeftConnections(Col,Row):
    weight = 10.0
    inhibWeight = -10
    #don't input off top
    if (Row == 0): return []
    #don't output off bottom
    if (Row == INPUT_NEURONS_HEIGHT-1): return []
    #don't output off bottom
    if (Col == 0): return []
    connector = []
    toNeuron = (Row)*50+Col
    fromNeuron = (Row*50)+Col
    connector = [(fromNeuron, toNeuron, inhibWeight, DELAY)]
    fromNeuron = (Row*50)+Col+1  #left
    connector = connector+ [(fromNeuron, toNeuron, weight, DELAY)]
    fromNeuron = ((Row-1)*50)+Col #below
    connector = connector+ [(fromNeuron, toNeuron, weight, DELAY)]
    return connector

#----------setup connections 3x3 on to or angle
#just a self connection
def set3x3OnToOrAngleConnections(Col,Row):
    weight = 10.0
    toNeuron = (Row)*50+Col
    fromNeuron = (Row)*50+Col
    connector = [(fromNeuron, toNeuron, weight, DELAY)]
    return connector

#----------setup connections 9x9 off to or angle
#---positive two down and from left (2). Inhibit self.
def set9x9OffToOrAngleConnections(Col,Row):
    weight = 7.0
    inhibWeight = -10
    #don't input off top
    if (Row <= 1): return []
    #don't input off left
    if (Col == 0): return []
    #don't input off right
    if (Col == INPUT_NEURONS_WIDTH): return []
    connector = []
    toNeuron = (Row)*50+Col
    fromNeuron = (Row)*50+Col
    connector = [(fromNeuron, toNeuron, inhibWeight, DELAY)]
    fromNeuron = ((Row-2)*50)+Col-1
    connector = connector+ [(fromNeuron, toNeuron, weight, DELAY)]
    fromNeuron = ((Row-2)*50)+Col
    connector = connector+ [(fromNeuron, toNeuron, weight, DELAY)]
    #fromNeuron = ((Row-2)*50)+Col+1
    #connector = connector+ [(fromNeuron, toNeuron, weight, DELAY)]
    return connector


